"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SdJwt = void 0;
const decode_1 = require("@sd-jwt/decode");
const present_1 = require("@sd-jwt/present");
const utils_1 = require("@sd-jwt/utils");
const jwt_1 = require("../jwt/jwt");
const keyBinding_1 = require("../keyBinding");
const disclosureFrame_1 = require("./disclosureFrame");
const disclosures_1 = require("./disclosures");
const error_1 = require("./error");
class SdJwt extends jwt_1.Jwt {
    constructor(options, additionalOptions) {
        super(options, additionalOptions);
        this.header = options === null || options === void 0 ? void 0 : options.header;
        this.payload = options === null || options === void 0 ? void 0 : options.payload;
        this.signature = options === null || options === void 0 ? void 0 : options.signature;
        this.disclosures = options === null || options === void 0 ? void 0 : options.disclosures;
        this.keyBinding = options === null || options === void 0 ? void 0 : options.keyBinding;
        if (additionalOptions === null || additionalOptions === void 0 ? void 0 : additionalOptions.hasherAndAlgorithm) {
            this.withHasher(additionalOptions.hasherAndAlgorithm);
        }
        if (additionalOptions === null || additionalOptions === void 0 ? void 0 : additionalOptions.saltGenerator) {
            this.withSaltGenerator(additionalOptions.saltGenerator);
        }
        if (additionalOptions === null || additionalOptions === void 0 ? void 0 : additionalOptions.disclosureFrame) {
            this.withDisclosureFrame(additionalOptions.disclosureFrame);
        }
        if (additionalOptions === null || additionalOptions === void 0 ? void 0 : additionalOptions.signer) {
            this.withSigner(additionalOptions.signer);
        }
    }
    /**
     *
     * Create an sd-jwt from a compact format. This will succeed for a normal jwt as well.
     *
     */
    static fromCompact(compact) {
        const { disclosures: d, keyBinding: kb, signature, payload, header } = (0, decode_1.sdJwtFromCompact)(compact);
        const disclosures = d === null || d === void 0 ? void 0 : d.map((disclosure) => new disclosures_1.Disclosure(disclosure.salt, disclosure.value, disclosure.key));
        const keyBinding = kb
            ? new keyBinding_1.KeyBinding()
                .withHeader(kb.header)
                .withPayload(kb.payload)
                .withSignature(kb.signature)
            : undefined;
        const sdJwt = new SdJwt({
            header,
            payload,
            signature,
            disclosures,
            keyBinding
        });
        sdJwt.compact = compact;
        return sdJwt;
    }
    /**
     *
     * Add a salt generator.
     *
     * Recommended size is 128 bits (i.e. 16 bytes).
     *
     * Salts will not be seeded and a new one will be used for each claim.
     *
     * @example
     *
     * Node.js: `crypto.randomBytes(128 / 8)`
     *
     * React Native: `expo-standard-web-crypto`
     *
     * Browser: `crypto.getRandomValues(new Uint8Array(128 / 8))`
     *
     */
    withSaltGenerator(saltGenerator) {
        this.saltGenerator = saltGenerator;
        return this;
    }
    /**
     *
     * Add a hasher that will be used to hash the disclosures.
     *
     * @note Make sure to return a base64url encoded version of the hash.
     *
     * @example
     *
     * Node.js: `createHash('sha256').update(input).digest().toString('base64url')`
     *
     */
    withHasher(hasherAndAlgorithm) {
        this.hasherAndAlgorithm = hasherAndAlgorithm;
        return this;
    }
    /**
     *
     * Adds the algorithm of the hasher to the payload.
     *
     * For convience, this also allows you to set the hasher.
     *
     * @throws when the hasher and algorithm are not set.
     *
     */
    addHasherAlgorithmToPayload(hasherAndAlgorithm) {
        if (hasherAndAlgorithm)
            this.withHasher(hasherAndAlgorithm);
        this.assertHashAndAlgorithm();
        this.addPayloadClaim('_sd_alg', this.hasherAndAlgorithm.algorithm);
        return this;
    }
    /**
     *
     * Set the `KeyBinding` jwt.
     *
     * This can be done as a holder to provide proof of possession of key material
     *
     */
    withKeyBinding(keyBinding) {
        const kb = typeof keyBinding === 'string'
            ? keyBinding_1.KeyBinding.fromCompact(keyBinding)
            : keyBinding instanceof keyBinding_1.KeyBinding
                ? keyBinding
                : keyBinding_1.KeyBinding.fromJwt(keyBinding);
        this.keyBinding = kb;
        return this;
    }
    /**
     *
     * Set the disclosure frame which will be applied via `SdJwt.applyDisclosureFrame` or when `SdJwt.toCompact` is called.
     *
     */
    withDisclosureFrame(disclosureFrame) {
        this.disclosureFrame = disclosureFrame;
        return this;
    }
    /**
     *
     * Apply the disclosure frame.
     *
     * @throws when the salt generator is not set
     * @throws when the hasher and algorithm is not set
     * @throws when the payload is not set
     * @throws when no disclosure frame is set
     * @throws when disclosures are included and a signature is set, but no signer is provided `*`
     * @throws when the disclosure frame is inconsistent with the payload
     *
     * * This is done as removing items from the payload alters the signature and it has to be resigned.
     *
     */
    applyDisclosureFrame() {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertSaltGenerator();
            this.assertHashAndAlgorithm();
            this.assertPayload();
            this.assertDisclosureFrame();
            if (this.disclosures &&
                this.disclosures.length > 0 &&
                this.signature &&
                !this.signer) {
                throw new error_1.SdJwtError('Signature is already set by the user when selectively disclosable items still have to be removed. This will invalidate the signature. Try to provide a signer on SdJwt.withSigner and SdJwt.toCompact will call it at the correct time.');
            }
            const { payload: framedPayload, disclosures } = yield (0, disclosureFrame_1.applyDisclosureFrame)(this.saltGenerator, this.hasherAndAlgorithm, this.addHasherAlgorithmToPayload().payload, this.disclosureFrame);
            this.disclosures = disclosures;
            this.payload = framedPayload;
        });
    }
    disclosuresWithDigest() {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertHashAndAlgorithm();
            if (!this.disclosures && this.disclosureFrame) {
                yield this.applyDisclosureFrame();
            }
            if (!this.disclosures)
                return undefined;
            return Promise.all(this.disclosures.map((d) => d.withCalculateDigest(this.hasherAndAlgorithm)));
        });
    }
    /**
     *
     * Assert that the disclosure frame is set.
     *
     */
    assertDisclosureFrame() {
        if (this.disclosureFrame)
            return;
        throw new error_1.SdJwtError('Disclosureframe must be defined');
    }
    /**
     *
     * Assert that the salt generator is set.
     *
     */
    assertSaltGenerator() {
        if (!this.saltGenerator) {
            throw new error_1.SdJwtError('Cannot create a disclosure without a salt generator. You can set it with this.withSaltGenerator()');
        }
    }
    /**
     *
     * Assert that the hasher and algorithm is set.
     *
     */
    assertHashAndAlgorithm() {
        if (!this.hasherAndAlgorithm) {
            throw new error_1.SdJwtError('A hasher and algorithm must be set in order to create a digests for disclosures or integrity protection of a kb-jwt. You can set it with this.withHasher()');
        }
    }
    /**
     *
     * Assert that a certain claim is included in the disclosure frame.
     *
     * @throws when the disclosure frame is not set
     *
     */
    assertClaimInDisclosureFrame(claimKey) {
        this.assertDisclosureFrame();
        const value = (0, utils_1.getValueByKeyAnyLevel)(this.disclosureFrame, claimKey);
        if (!value) {
            throw new error_1.SdJwtError(`Claim key '${claimKey}' not found in any level of the disclosureFrame`);
        }
    }
    /**
     * This function creates a presentation of an SD-JWT, based on the presentation frame. The
     * presentation frame is similar to the disclosure frame, and allows you to present a subset
     * of the disclosures.
     *
     * If no `presentationFrame` is passed, the entire SD-JWT will be presented.
     * To create a presentation without any of the disclosures, pass an empty object as the `presentationFrame`.
     *
     * @example
     * The following example will expose `name`, `a.nested`, and `orderItems[0]` and `orderItems[2]`.
     * Based on the disclosures it will also expose the parent and child disclosures when needed.
     * E.g. if `a` can only be disclosed as a whole, disclosing `a.nested` will also disclose `a`.
     * The same is true for child disclosures. If you expose `name`, and it potentially contains recursive
     * disclosures, all disclosures under name will be disclosed as well.
     * ```ts
     * await sdJwt.present({
     *   name: true,
     *   a: {
     *     nested: 'property'
     *   }
     *   orderItems: [true, false, true]
     * })
     * ```
     *
     * @throws when the presentation frame does not match the decoded/pretty payload of the sd-jwt
     * @throws when the presentation frame contains fields other than object, array or boolean
     *
     */
    present(presentationFrame) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.disclosures && this.disclosureFrame) {
                yield this.applyDisclosureFrame();
            }
            // TODO: wouldn't it be easier if this returned the value so we don't have to use !
            this.assertHashAndAlgorithm();
            this.assertPayload();
            // If no presentationFrame is passed, we want to disclose everything
            if (!presentationFrame) {
                return yield this.__toCompact(this.disclosures, false);
            }
            if ((Object.keys(presentationFrame).length > 0 && !this.disclosures) ||
                ((_a = this.disclosures) === null || _a === void 0 ? void 0 : _a.length) === 0) {
                throw new error_1.SdJwtError('Cannot create a presentation with disclosures while no disclosures are on the sd-jwt');
            }
            // Calculate the digests for all disclosures
            const disclosuresWithDigest = yield this.disclosuresWithDigest();
            const requiredDisclosures = (0, present_1.getDisclosuresForPresentationFrame)(this.payload, presentationFrame, yield this.getPrettyClaims(), disclosuresWithDigest === null || disclosuresWithDigest === void 0 ? void 0 : disclosuresWithDigest.map((d) => d.asJson()));
            return yield this.__toCompact(requiredDisclosures.map((d) => disclosures_1.Disclosure.fromJson(d)), false);
        });
    }
    /**
     *
     * Verify the sd-jwt.
     *
     * It validates the following properties:
     *   - sd-jwt issuer signature
     *   - Optionally, the required claims
     *   - The `nbf` and `exp` claims
     *   - Whether the key binding is valid
     *
     */
    verify(verifier, requiredClaimKeys, kbJwtPublicKeyJwk, issuerPublicKeyJwk) {
        const _super = Object.create(null, {
            verify: { get: () => super.verify }
        });
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.assertSignature();
            if (this.keyBinding) {
                this.assertHashAndAlgorithm();
            }
            const jwtVerificationResult = (yield _super.verify.call(this, verifier, requiredClaimKeys, issuerPublicKeyJwk));
            if (this.keyBinding) {
                if (!this.keyBinding.expectedSdHash) {
                    // Calculate and set expected _sd_hash
                    const sdHash = yield this.calculateSdHash();
                    this.keyBinding.withExpectedSdHash(sdHash);
                }
                const { isValid } = yield this.keyBinding.verify(verifier, [], kbJwtPublicKeyJwk);
                jwtVerificationResult.isKeyBindingValid = isValid;
            }
            const claimKeys = (0, utils_1.getAllKeys)(this.payload).concat(((_a = this.disclosures) !== null && _a !== void 0 ? _a : []).map((d) => d.decoded[1]));
            if (requiredClaimKeys) {
                jwtVerificationResult.areRequiredClaimsIncluded =
                    requiredClaimKeys.every((key) => claimKeys.includes(key));
            }
            return Object.assign(Object.assign({}, jwtVerificationResult), { isValid: Object.entries(jwtVerificationResult)
                    .filter(([key, value]) => typeof value === 'boolean' && key !== 'isValid')
                    .every(([, value]) => !!value) });
        });
    }
    /**
     *
     * Utility method to check whether the expected hasher algorithm is used.
     *
     */
    checkHasher(expectedHasher) {
        try {
            this.assertPayload();
            this.assertClaimInPayload('_sd_alg', expectedHasher.toString());
            return true;
        }
        catch (e) {
            console.error(e);
            return false;
        }
    }
    assertNonSelectivelyDisclosableClaim(claimKey) {
        try {
            this.assertClaimInDisclosureFrame(claimKey);
        }
        catch (error) {
            // Check if it's an SdJwtError (to prevent other errors making the method not throw)
            if (error instanceof error_1.SdJwtError)
                return;
        }
        throw new error_1.SdJwtError(`Claim key '${claimKey}' was found in the disclosure frame. This claim is not allowed to be selectively disclosed`);
    }
    assertNonSelectivelyDisclosableClaims() {
        if (!this.disclosureFrame)
            return;
        // NOTE: we don't include any properties here as it is a SHOULD
        // in the spec, and not a MUST
        for (const claimKey of ['_sd', '_sd_alg', '...']) {
            this.assertNonSelectivelyDisclosableClaim(claimKey);
        }
    }
    /**
     *
     * Return all claims from the payload and the disclosures on their original place.
     *
     */
    getPrettyClaims() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.assertPayload();
            this.assertHashAndAlgorithm();
            const disclosuresWithDigest = yield this.disclosuresWithDigest();
            const newPayload = (0, decode_1.decodeDisclosuresInPayload)(this.payload, (_a = disclosuresWithDigest === null || disclosuresWithDigest === void 0 ? void 0 : disclosuresWithDigest.map((d) => d.asJson())) !== null && _a !== void 0 ? _a : []);
            return newPayload;
        });
    }
    /**
     *
     * Create a compact format of the sd-jwt.
     *
     * This will
     *   - Apply the disclosure frame
     *   - Add a signature if there is none
     *
     * @throws When the signature and signer are not defined
     * @throws When a claim is requested to be selectively disclosable, but it was not found in the payload
     *
     */
    toCompact() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.__toCompact();
        });
    }
    __toCompact(disclosures = this.disclosures, shouldApplyFrame = true) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            this.assertHeader();
            this.assertPayload();
            if (this.keyBinding && !this.keyBinding.expectedSdHash) {
                this.assertHashAndAlgorithm();
            }
            if (this.disclosureFrame && shouldApplyFrame) {
                yield this.applyDisclosureFrame();
            }
            disclosures !== null && disclosures !== void 0 ? disclosures : (disclosures = this.disclosures);
            const compactHeader = utils_1.Base64url.encode(JSON.stringify(this.header));
            const compactPayload = utils_1.Base64url.encode(JSON.stringify(this.payload));
            const sSignature = this.signature
                ? utils_1.Base64url.encode(this.signature)
                : utils_1.Base64url.encode((yield this.signAndAdd()).signature);
            const sDisclosures = disclosures && disclosures.length > 0
                ? `~${disclosures.join('~')}~`
                : '~';
            const sdJwtWithoutKb = `${compactHeader}.${compactPayload}.${sSignature}${sDisclosures}`;
            if (this.keyBinding) {
                const sdHash = (_a = this.keyBinding.expectedSdHash) !== null && _a !== void 0 ? _a : (yield this.calculateSdHash(sdJwtWithoutKb));
                if (this.keyBinding.signature &&
                    !((_b = this.keyBinding.payload) === null || _b === void 0 ? void 0 : _b._sd_hash)) {
                    throw new error_1.SdJwtError("Key binding is already signed, but missing _sd_hash. If you're manually signing the kb-jwt, make sure the correct _sd_hash is set.");
                }
                // If the signature is already set we don't want to add the _sd_hash ourselves
                // Also the signature won't be re-calculated if it's already set -- this seems like a security issue to me
                if (!this.keyBinding.signature &&
                    !((_c = this.keyBinding.payload) === null || _c === void 0 ? void 0 : _c._sd_hash)) {
                    this.keyBinding.withSdHashClaim(sdHash);
                }
                // Make sure the sd_hash is valid. If there's already a signature set
                // this will ensure the signature was made with the correct _sd_hash
                yield this.keyBinding.assertValidForKeyBinding(sdHash);
            }
            const kb = (_e = (yield ((_d = this.keyBinding) === null || _d === void 0 ? void 0 : _d.toCompact()))) !== null && _e !== void 0 ? _e : '';
            return sdJwtWithoutKb + kb;
        });
    }
    calculateSdHash(compact) {
        return __awaiter(this, void 0, void 0, function* () {
            this.assertHashAndAlgorithm();
            const c = compact !== null && compact !== void 0 ? compact : this.compact;
            if (!c) {
                throw new error_1.SdJwtError("Unable to calculate _sd_hash for sd-jwt, without 'compact' variant to compare _sd_hash. Use `fromCompact`, or call `sdJwt.keyBinding.withExpectedSdHash` to set the expected _sd_hash.");
            }
            return (0, decode_1.calculateSdHash)(c, this.hasherAndAlgorithm);
        });
    }
}
exports.SdJwt = SdJwt;
//# sourceMappingURL=sdJwt.js.map