"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SdJwtVc = void 0;
const sdJwt_1 = require("../sdJwt");
const sdJwt_2 = require("../sdJwt");
const jwt_1 = require("../jwt");
const decode_1 = require("@sd-jwt/decode");
const keyBinding_1 = require("../keyBinding");
class SdJwtVc extends sdJwt_2.SdJwt {
    assertNonSelectivelyDisclosableClaims() {
        if (!this.disclosureFrame)
            return;
        const nonSelectivelyDisclosableClaims = [
            'iss',
            'iat',
            'nbf',
            'exp',
            'cnf',
            'vct',
            'status'
        ];
        for (const claimKey of nonSelectivelyDisclosableClaims) {
            this.assertNonSelectivelyDisclosableClaim(claimKey);
        }
    }
    validateSdJwtVc(expectedCnfClaim) {
        try {
            this.assertNonSelectivelyDisclosableClaims();
            this.assertHeader();
            this.assertPayload();
            this.assertClaimInHeader('typ', 'vc+sd-jwt');
            this.assertClaimInHeader('alg');
            this.assertClaimInPayload('iss');
            this.assertClaimInPayload('vct');
            this.assertClaimInPayload('iat');
            if (expectedCnfClaim) {
                this.assertClaimInPayload('cnf', expectedCnfClaim);
            }
        }
        catch (e) {
            if (e instanceof Error) {
                e.message = `jwt is not valid for usage with sd-jwt-vc. Error: ${e.message}`;
            }
            throw e;
        }
    }
    /**
     *
     * Instantiate a sd-jwt-vc from a compact format.
     *
     * @throws when the compact sd-jwt-vc is not a valid sd-jwt-vc
     *
     */
    static fromCompact(compact) {
        const { disclosures: d, keyBinding: kb, signature, payload, header } = (0, decode_1.sdJwtVcFromCompact)(compact);
        const disclosures = d === null || d === void 0 ? void 0 : d.map((disclosure) => new sdJwt_1.Disclosure(disclosure.salt, disclosure.value, disclosure.key));
        const keyBinding = kb
            ? new keyBinding_1.KeyBinding()
                .withHeader(kb.header)
                .withPayload(kb.payload)
                .withSignature(kb.signature)
            : undefined;
        const sdJwtVc = new SdJwtVc({
            header,
            payload,
            signature,
            disclosures,
            keyBinding
        });
        sdJwtVc.compact = compact;
        return sdJwtVc;
    }
    /**
     *
     * Verify the sd-jwt-vc.
     *
     * It validates the following properties:
     *   - sd-jwt issuer signature
     *   - Optionally, the required claims
     *   - The `nbf` and `exp` claims
     *   - Whether the key binding is valid
     *   - Whether the expected key binding is used
     *   - Whether the required sd-jwt-vc properties are included
     *
     */
    verify(verifier, requiredClaimKeys, expectedCnfClaim, kbJwtPublicKeyJwk, issuerPublicKeyJwk) {
        const _super = Object.create(null, {
            verify: { get: () => super.verify }
        });
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const kbJwtPublicKeyJwkToUse = kbJwtPublicKeyJwk !== null && kbJwtPublicKeyJwk !== void 0 ? kbJwtPublicKeyJwk : (_b = (_a = this.payload) === null || _a === void 0 ? void 0 : _a.cnf) === null || _b === void 0 ? void 0 : _b.jwk;
            const sdJwtVerificationResult = (yield _super.verify.call(this, verifier, requiredClaimKeys, kbJwtPublicKeyJwkToUse, issuerPublicKeyJwk));
            try {
                sdJwtVerificationResult.containsRequiredVcProperties = true;
                this.validateSdJwtVc(expectedCnfClaim);
                if (expectedCnfClaim) {
                    sdJwtVerificationResult.containsExpectedKeyBinding = true;
                }
            }
            catch (e) {
                if (e instanceof jwt_1.JwtError &&
                    e.message ==
                        "jwt is not valid for usage with sd-jwt-vc. Error: Claim key 'cnf' was found, but values did not match within the payload") {
                    sdJwtVerificationResult.containsExpectedKeyBinding = false;
                }
                else {
                    sdJwtVerificationResult.containsRequiredVcProperties = false;
                }
                // The verification result is not valid if an error occurred
                sdJwtVerificationResult.isValid = false;
            }
            return sdJwtVerificationResult;
        });
    }
    /**
     *
     * Create a compact format of the sd-jwt-vc.
     *
     * This will
     *   - Apply the disclosure frame
     *   - Add a signature if there is none
     *
     * @throws when the sd-jwt-vc is not conformant to the specification
     * @throws When the signature and signer are not defined
     * @throws When a claim is requested to be selectively disclosable, but it was not found in the payload
     *
     */
    toCompact() {
        const _super = Object.create(null, {
            toCompact: { get: () => super.toCompact }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.validateSdJwtVc();
            return yield _super.toCompact.call(this);
        });
    }
}
exports.SdJwtVc = SdJwtVc;
//# sourceMappingURL=sdJwtVc.js.map